﻿-- ShunkIt.lua
-- $Id: ShunkIt.lua 10 2009-09-01 02:24:55Z LaoTseu $

ShunkIt = ShunkIt or {}
ShunkIt.revision = ShunkIt.revision or {} 
ShunkIt.revision.main	= ("$Revision: 10 $"):match("(%d+)")
ShunkIt.version      	= GetAddOnMetadata("ShunkIt", "Version"):match("([^ ]+)")
ShunkIt.revision.toc  	= GetAddOnMetadata("ShunkIt", "Version"):match("%$Revision:%s(%d+)")

----------------------
--      Locals      --
----------------------

local L = ShunkIt.localized
--local defaults, defaultsPC, db, dbpc = {}, {}
local UPDATEPERIOD, elapsed = 0.5, 0
local need_scan, busy, delay_scan = nil, nil, nil

local item_to_shunk, icon, shunk_count = nil, nil, 0
local shunk_table = {}
local shunk_bag, shunk_slot = 1, 1
local default_icon = "Interface\\Icons\\INV_Misc_Shell_03"

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("ShunkIt", 
											 {
												type = "data source", 
											  	text = "",
											  	icon = default_icon
})


------------------------------
--      Util Functions      --
------------------------------

local function Print(...) SELECTED_CHAT_FRAME:AddMessage(string.join(" ", "|cFF33FF99" .. L["ShunkIt"] .. "|r:", ...)) end

local debugf = tekDebug and tekDebug:GetFrame("ShunkIt")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

------------------------------
--      Core Functions     --
------------------------------

CreateFrame("GameTooltip", "ShunkItTooltip")
ShunkItTooltip:SetOwner(WorldFrame, "ANCHOR_NONE")

ShunkItTooltip:AddFontStrings(
    ShunkItTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
    ShunkItTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
)

local cache = {}
SunkIt_cache = cache
local function IsOpenable(bag, slot, itemLink)
	--local _,_,itemID = string.find(itemLink, "^[|]c%x+[|]H[^:]+[:]([^:]+)[:].*[|]h%[.+%]")
	--local found,_,itemID = string.find(itemLink, "item[:](%d+)[:]")

	--assert(found,"No itemID: " .. gsub(itemLink, "\124", "\124\124"))
	
	if not bag or not slot then return false end
	if not itemLink then itemLink = GetContainerItemLink(bag, slot) end
	
	if cache[itemLink] ~= nil then return cache[itemLink] end
	
	ShunkItTooltip:ClearLines()
	if not ShunkItTooltip:IsOwned(WorldFrame) then ShunkItTooltip:SetOwner(WorldFrame, "ANCHOR_NONE") end
 	ShunkItTooltip:SetBagItem(bag,slot)
	
	for i=1,ShunkItTooltip:NumLines() do
	   local mytext = getglobal("ShunkItTooltipTextLeft" .. i)
	   local text = mytext:GetText()

		if text == ITEM_OPENABLE or text == L["Use: Open the clam!"] then
			cache[itemLink] = true
			return true
		elseif text == LOCKED then
			-- Do not cache the result for locked items, they may become unlocked
			return false
		end
	end
	
	cache[itemLink] = false
	return false

end

local function ScanBags()

	item_to_shunk, shunk_bag, shunk_slot = nil, nil, nil
	shunk_count = 0
	for k in pairs(shunk_table) do shunk_table[k] = nil end

	local newIcon = nil
	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do

			local itemLink = GetContainerItemLink(bag, slot)
			if itemLink and IsOpenable(bag, slot, itemLink) then
				local _,_,itemID = string.find(itemLink, "item[:](%d+)[:]")
				shunk_table[itemID] = (shunk_table[itemID] or 0) + tonumber(select(2,GetContainerItemInfo(bag, slot)))
				if not item_to_shunk then
					item_to_shunk = GetItemInfo(itemLink)
					shunk_bag, shunk_slot = bag, slot
					newIcon = GetContainerItemInfo(bag, slot)
				end
			elseif itemLink and not GetItemInfo(itemLink) then
				-- This is a workaround for a intermitend situation when opening
				-- multiple items of the same kind in a row sometimes result in
				-- the item information being screewed up.
				-- If this happen, we delay de scan until the information is realiable
				delay_scan = true
				return
			end

		end
	end
	
	for k in pairs (shunk_table) do
		shunk_count = shunk_count + shunk_table[k]
	end
	icon = newIcon or default_icon
	need_scan = nil
end

local function myOnClick(self, button, down) 
	if not InCombatLockdown() and not busy and IsOpenable(shunk_bag, shunk_slot) then
		UseContainerItem(shunk_bag, shunk_slot)
		return true
	else
		return false
	end
end

-----------------------------
--      Event Handler      --
-----------------------------

local f = CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
f:RegisterEvent("ADDON_LOADED")


function f:ADDON_LOADED(event, addon)
	if addon ~= "ShunkIt" then return end

	--AddonTemplateDB, AddonTemplateDBPC = setmetatable(AddonTemplateDB or {}, {__index = defaults}), setmetatable(AddonTemplateDBPC or {}, {__index = defaultsPC})
	--db, dbpc = AddonTemplateDB, AddonTemplateDBPC

	-- Do anything you need to do after addon has loaded

	--LibStub("tekKonfig-AboutPanel").new("ShunkIt", "ShunkIt") -- Remove first arg if no parent config panel

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil

	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end


function f:PLAYER_LOGIN()
	self:RegisterEvent("PLAYER_LOGOUT")

	-- Register events
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

	-- Start the OnUpdate 
	self:SetScript("OnUpdate", function(self, elap)
		elapsed = elapsed + elap
		
		if delay_scan then 
			-- Workaround for when good item information is not available and we
			-- need to wait a bit
			elapsed = UPDATEPERIOD/2
			need_scan = true
			delay_scan = nil
			return
		end
		
		if InCombatLockdown() or busy or not need_scan or elapsed < UPDATEPERIOD then return end

		elapsed = 0

		-- Find object to Shunk
		ScanBags()
		
		-- Adjust the display
		if shunk_count > 0 then
			dataobj.text = string.format(L["%s - %d"], item_to_shunk, shunk_count)
		else
			dataobj.text = ""
		end
		dataobj.icon = icon
	end)
	
	-- Set the OnClick function for the data broker
	dataobj.OnClick = myOnClick
	
	-- Find the revision number
	if ShunkIt.revision then
		ShunkIt.rev = 0
		for _, rev in pairs(ShunkIt.revision) do
			local revision = tonumber(rev)
			if revision and revision > ShunkIt.rev then ShunkIt.rev = revision end
		end
	else
		assert(false,"No ShunkIt.revision")
	end

	
	need_scan = true

	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end


function f:PLAYER_LOGOUT()
	--for i,v in pairs(defaults) do if db[i] == v then db[i] = nil end end
	--for i,v in pairs(defaultsPC) do if dbpc[i] == v then dbpc[i] = nil end end

	-- Do anything you need to do as the player logs out
end

function f:BAG_UPDATE()
	elapsed = 0 -- Let's wait a bit
	need_scan = true
end

function f:UNIT_SPELLCAST_SENT()
	busy = true
end

function f:UNIT_SPELLCAST_FAILED()
	busy = nil
end

function f:UNIT_SPELLCAST_STOP()
	busy = nil
end

function f:UNIT_SPELLCAST_INTERRUPTED()
	busy = nil
end

function f:UNIT_SPELLCAST_SUCCEEDED()
	busy = nil
end




-----------------------------
--      Slash Handler      --
-----------------------------

SLASH_ADDONTEMPLATE1 = "/shunkit"
SlashCmdList.ADDONTEMPLATE = function(msg)
	if not myOnClick() then
		Print(L["Nothing to do or in combat"])
	end
end

-----------------------------
--        LDB Stuff        --
-----------------------------

function dataobj:OnTooltipShow()
	self:AddLine(format("%s v%s (r%d)", L["ShunkIt"], ShunkIt.version, ShunkIt.rev))
	if shunk_count == 0 then return end
	
	self:AddLine(' ')
	self:AddLine(format(L["%d items to open"],shunk_count))
	for link, count in pairs (shunk_table) do
		local name, _, _, _, _, _, _, _, _, texture = GetItemInfo(link)
		self:AddLine(format(L["%s x %d"],name,count),1,1,1)
		self:AddTexture(texture)
	end
end

